LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY tb_divider_simple IS
  Generic(
    N : integer := 8;
    M : integer := 8
    );
END tb_divider_simple;

ARCHITECTURE behavior OF tb_divider_simple IS

  -- Component Declaration for the Unit Under Test (UUT)

  COMPONENT divider
    Generic (N : integer := 8;
             M : integer := 8);
    PORT(
      dividend : IN  signed(N-1 downto 0);
      divisor : IN  signed(M-1 downto 0);
      valid : IN  std_logic;
      quotient : OUT  signed(N-1 downto 0);
      remainder : OUT  unsigned(M-1 downto 0);
      out_valid : OUT  std_logic;
      busy : OUT  std_logic;
      err : OUT  std_logic;
      rst : IN  std_logic;
      clk : IN  std_logic
      );
  END COMPONENT;

  --Inputs
  signal dividend : signed(N-1 downto 0) := (others => '0');
  signal divisor : signed(M-1 downto 0) := (others => '0');
  signal valid : std_logic := '0';
  signal rst : std_logic := '0';
  signal clk : std_logic := '0';

  --Outputs
  signal quotient : signed(N-1 downto 0);
  signal remainder : unsigned(M-1 downto 0);
  signal out_valid : std_logic;
  signal busy : std_logic;
  signal err : std_logic;

  -- Clock period definitions
  constant clk_period : time := 10 ns;

BEGIN

  -- Instantiate the Unit Under Test (UUT)
  uut: divider
    Generic MAP (
      N => 8,
      M => 8
      )
    PORT MAP (
      dividend => dividend,
      divisor => divisor,
      valid => valid,
      quotient => quotient,
      remainder => remainder,
      out_valid => out_valid,
      busy => busy,
      err => err,
      rst => rst,
      clk => clk
      );

  -- Clock process definitions
  clk_process :process
  begin
    clk <= '0';
    wait for clk_period/2;
    clk <= '1';
    wait for clk_period/2;
  end process;


  -- Stimulus process
  stim_proc: process
  begin
    rst <= '1';
    -- hold reset state for 100 ns.
    wait for 100 ns;
    rst <= '0';

    -- Simple tests:
    -- 100/9, should output quotient = 11, remainder = 1
    dividend <= to_signed(100,8);
    divisor  <= to_signed(9,8);
    valid    <= '1';
    wait for clk_period;
    valid    <= '0';

    -- 120/-7, should output quotient = -17, remainder = 1
    wait for clk_period*(N+1);
    dividend <= to_signed(120,8);
    divisor  <= to_signed(-7,8);
    valid    <= '1';
    wait for clk_period;
    valid    <= '0';

    wait for clk_period*(N+1);
    wait for clk_period*10;
    report "Simulation finished succesfully" severity failure;
    wait;
  end process;

END;
